class Base32Hex {
    static encode(str) {
        function toBytes(str) {
            return str.split("").map((s, i) => Base32Hex._pad(str.charCodeAt(i).toString(2))).join("")
        }
        function split(str, token = [], size = 5) {
            return str.length ? token.concat(split(str.substring(size, str.length), [Base32Hex._pad(str.substring(0, 5), 5, false)])) : token
        }
        function binary(str) {
            return parseInt(str, 2)
        }
        function getPadSize(str, len = 8) {
            return len * Math.ceil((str.length / len))
        }
        function map(tokens) {
            let str = tokens.map(token => Base32Hex._codeMap()[binary(token)]).join("")
            return Base32Hex._pad(str, getPadSize(str, 8), false, '=')
        }
        return map(split(toBytes(str)))
    }
    static decode(str) {
        function trim(str) {
            return str.match(/^(.+)[^\=]?\b/)[1]
        }
        function binary(str) {
            return str.toString(2)
        }
        function demap(str) {
            return trim(str).split("").map(s => Base32Hex._pad(binary(Base32Hex._codeMap().indexOf(s)), 5, true))
        }
        function getTrimSize(str) {
            return 8 * Math.round(str.length / 8)
        }
        function merge(tokens) {
            let str = tokens.join("")
            return str.substring(0, getTrimSize(str))
        }
        function fromBytes(str, result = "") {
            return str.length < 8 ? result : fromBytes(str.substring(8, str.length), result + String.fromCharCode(parseInt(str.substring(0, 8), 2)))
        }
        return fromBytes(merge(demap(str)))
    }
    static sort(dataset, asc = true) { return dataset.sort((a, b) => Base32Hex._sort(a, b, asc)) }
    static _codeMap() { return ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V'] }
    static _pad(str, len = 8, front = true, code = '0') { return front ? code.repeat(len - str.length) + str : str + code.repeat(len - str.length) }
    static _length(str) { return str.match(/^(.+)[^\=]?\b/)[1].length }
    static _sort(a, b, asc = true) {
        let diff = Base32Hex._length(a) - Base32Hex._length(b);
        return (diff ? diff : a < b ? -1 : a > b ? 1 : 0) * (asc ? 1 : -1)
    }
}
module.exports = Base32Hex